#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 12:29, 20/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from numpy.random import seed, permutation
from numpy import dot, ones
from opfunu.cec.cec2010.utils import *


def F1(solution=None, name="Shifted Elliptic Function", shift_data_file="f01_o.txt"):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    z = solution - shift_data
    return f2_elliptic__(z)


def F2(solution=None, name="Shifted Rastrigin’s Function", shift_data_file="f02_o.txt"):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    z = solution - shift_data
    return f3_rastrigin__(z)


def F3(solution=None, name="Shifted Ackley’s Function", shift_data_file="f03_o.txt"):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    z = solution - shift_data
    return f4_ackley__(z)


def F4(solution=None, name="Single-group Shifted and m-rotated Elliptic Function", shift_data_file="f04_op.txt", matrix_data_file="f04_m.txt", m_group=50):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    matrix = load_matrix_data__(matrix_data_file)
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1,:].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    idx1 = permu_data[:m_group]
    idx2 = permu_data[m_group:]
    z_rot_elliptic = dot(z[idx1], matrix[:m_group, :m_group])
    z_elliptic = z[idx2]
    return f2_elliptic__(z_rot_elliptic) * 10**6 + f2_elliptic__(z_elliptic)


def F5(solution=None, name="Single-group Shifted and m-rotated Rastrigin’s Function", shift_data_file="f05_op.txt", matrix_data_file="f05_m.txt", m_group=50):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    matrix = load_matrix_data__(matrix_data_file)
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    idx1 = permu_data[:m_group]
    idx2 = permu_data[m_group:]
    z_rot_rastrigin = dot(z[idx1], matrix[:m_group, :m_group])
    z_rastrigin = z[idx2]
    return f3_rastrigin__(z_rot_rastrigin) * 10 ** 6 + f3_rastrigin__(z_rastrigin)


def F6(solution=None, name="Single-group Shifted and m-rotated Ackley’s Function", shift_data_file="f06_op.txt", matrix_data_file="f06_m.txt", m_group=50):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    matrix = load_matrix_data__(matrix_data_file)
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    idx1 = permu_data[:m_group]
    idx2 = permu_data[m_group:]
    z_rot_ackley = dot(z[idx1], matrix[:m_group, :m_group])
    z_ackley = z[idx2]
    return f4_ackley__(z_rot_ackley) * 10 ** 6 + f4_ackley__(z_ackley)


def F7(solution=None, name="Single-group Shifted m-dimensional Schwefel’s Problem 1.2", shift_data_file="f07_op.txt", m_group=50):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    idx1 = permu_data[:m_group]
    idx2 = permu_data[m_group:]
    z_schwefel = z[idx1]
    z_shpere = z[idx2]
    return f5_schwefel__(z_schwefel) * 10 ** 6 + f1_sphere__(z_shpere)


def F8(solution=None, name=" Single-group Shifted m-dimensional Rosenbrock’s Function", shift_data_file="f08_op.txt", m_group=50):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    idx1 = permu_data[:m_group]
    idx2 = permu_data[m_group:]
    z_rosenbrock = z[idx1]
    z_sphere = z[idx2]
    return f6_rosenbrock__(z_rosenbrock) * 10 ** 6 + f1_sphere__(z_sphere)


def F9(solution=None, name="D/2m-group Shifted and m-rotated Elliptic Function", shift_data_file="f09_op.txt", matrix_data_file="f09_m.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / (2 * m_group))
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size/(2*m_group) <= 1:
        print("CEC 2020 F9 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    matrix = load_matrix_data__(matrix_data_file)
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i*m_group:(i+1)*m_group]
        z1 = dot(z[idx1], matrix[:len(idx1), :len(idx1)])
        result += f2_elliptic__(z1)
    idx2 = permu_data[int(problem_size/2):problem_size]
    z2 = z[idx2]
    result += f2_elliptic__(z2)
    return result


def F10(solution=None, name="D/2m-group Shifted and m-rotated Rastrigin’s Function", shift_data_file="f10_op.txt", matrix_data_file="f10_m.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / (2 * m_group))
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size / (2 * m_group) <= 1:
        print("CEC 2020 F10 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    matrix = load_matrix_data__(matrix_data_file)
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i * m_group:(i + 1) * m_group]
        z1 = dot(z[idx1], matrix[:len(idx1), :len(idx1)])
        result += f3_rastrigin__(z1)
    idx2 = permu_data[int(problem_size / 2):problem_size]
    z2 = z[idx2]
    result += f3_rastrigin__(z2)
    return result


def F11(solution=None, name="D/2m-group Shifted and m-rotated Ackley’s Function", shift_data_file="f11_op.txt", matrix_data_file="f11_m.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / (2 * m_group))
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size / (2 * m_group) <= 1:
        print("CEC 2020 F11 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    matrix = load_matrix_data__(matrix_data_file)
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i * m_group:(i + 1) * m_group]
        z1 = dot(z[idx1], matrix[:len(idx1), :len(idx1)])
        result += f4_ackley__(z1)
    idx2 = permu_data[int(problem_size / 2):problem_size]
    z2 = z[idx2]
    result += f4_ackley__(z2)
    return result


def F12(solution=None, name="D/2m-group Shifted m-dimensional Schwefel’s Problem 1.2", shift_data_file="f12_op.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / (2 * m_group))
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size / (2 * m_group) <= 1:
        print("CEC 2020 F12 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i * m_group:(i + 1) * m_group]
        result += f5_schwefel__(z[idx1])
    idx2 = permu_data[int(problem_size / 2):problem_size]
    result += f1_sphere__(z[idx2])
    return result


def F13(solution=None, name="D/2m-group Shifted m-dimensional Rosenbrock’s Function", shift_data_file="f13_op.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / (2 * m_group))
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size / (2 * m_group) <= 1:
        print("CEC 2020 F12 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i * m_group:(i + 1) * m_group]
        result += f6_rosenbrock__(z[idx1])
    idx2 = permu_data[int(problem_size / 2):problem_size]
    result += f1_sphere__(z[idx2])
    return result


def F14(solution=None, name="D/2m-group Shifted and m-rotated Elliptic Function", shift_data_file="f14_op.txt", matrix_data_file="f14_m.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / m_group)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size / m_group <= 1:
        print("CEC 2020 F14 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    matrix = load_matrix_data__(matrix_data_file)
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i * m_group:(i + 1) * m_group]
        result += f2_elliptic__(dot(z[idx1], matrix))
    return result


def F15(solution=None, name="D/2m-group Shifted and m-rotated Rastrigin’s Function", shift_data_file="f15_op.txt", matrix_data_file="f15_m.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / m_group)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size / m_group <= 1:
        print("CEC 2020 F15 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    matrix = load_matrix_data__(matrix_data_file)
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i * m_group:(i + 1) * m_group]
        result += f3_rastrigin__(dot(z[idx1], matrix))
    return result


def F16(solution=None, name="D/2m-group Shifted and m-rotated Ackley’s Function", shift_data_file="f16_op.txt", matrix_data_file="f16_m.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / m_group)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size / m_group <= 1:
        print("CEC 2020 F16 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    matrix = load_matrix_data__(matrix_data_file)
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i * m_group:(i + 1) * m_group]
        result += f4_ackley__(dot(z[idx1], matrix))
    return result


def F17(solution=None, name="D/2m-group Shifted m-dimensional Schwefel’s Problem 1.2", shift_data_file="f17_op.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / m_group)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size / m_group <= 1:
        print("CEC 2020 F17 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i * m_group:(i + 1) * m_group]
        result += f5_schwefel__(z[idx1])
    return result


def F18(solution=None, name="D/2m-group Shifted m-dimensional Rosenbrock’s Function", shift_data_file="f18_op.txt", m_group=50):
    problem_size = len(solution)
    epoch = int(problem_size / m_group)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    if problem_size / m_group <= 1:
        print("CEC 2020 F18 not support {}. You can change m_group smaller or problem size larger!!!".format(problem_size))
        return 1
    op_data = load_matrix_data__(shift_data_file)
    if problem_size == 1000:
        shift_data = op_data[:1, :].reshape(-1)
        permu_data = (op_data[1:, :].reshape(-1) - ones(problem_size)).astype(int)
    else:
        seed(0)
        shift_data = op_data[:1, :].reshape(-1)[:problem_size]
        permu_data = permutation(problem_size)
    z = solution - shift_data
    result = 0.0
    for i in range(0, epoch):
        idx1 = permu_data[i * m_group:(i + 1) * m_group]
        result += f6_rosenbrock__(z[idx1])
    return result


def F19(solution=None, name="Shifted Schwefel’s Problem 1.2", shift_data_file="f19_o.txt"):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    z = solution - shift_data
    return f5_schwefel__(z)


def F20(solution=None, name="Shifted Rosenbrock’s Function", shift_data_file="f20_o.txt"):
    problem_size = len(solution)
    if problem_size > 1000:
        print("CEC 2010 not support for problem size > 1000")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    z = solution - shift_data
    return f6_rosenbrock__(z)



#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 14:21, 26/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
# -------------------------------------------------------------------------------------------------------%

from copy import deepcopy
from numpy import sum, exp, cos, sin, sqrt, e, pi, abs, round, log, power, fill_diagonal, zeros


def ackley__(solution=None):
    solution = solution.reshape((-1))
    return -20 * exp(-0.2 * sqrt(sum(solution ** 2) / len(solution))) - exp(sum(cos(2 * pi * solution)) / len(solution)) + 20 + e


def bent_cigar__(solution=None):
    solution = solution.reshape((-1))
    return solution[0] ** 2 + 10 ** 6 * sum(solution[1:] ** 2)


def discus__(solution=None):
    solution = solution.reshape((-1))
    return 10 ** 6 * solution[0] ** 2 + sum(solution[1:] ** 2)


def weierstrass__(solution=None, a=0.5, b=3, k_max=20):
    solution = solution.reshape((-1))
    result = 0.0
    for i in range(0, len(solution)):
        t1 = sum([a ** k * cos(2 * pi * b ** k * (solution[i] + 0.5)) for k in range(0, k_max)])
        result += t1
    t2 = len(solution) * sum([a ** k * cos(2 * pi * b ** k * 0.5) for k in range(0, k_max)])
    return result - t2


def katsuura__(solution=None):
    solution = solution.reshape((-1))
    result = 1.0
    for i in range(0, len(solution)):
        t1 = sum([abs(2 ** j * solution[i] - round(2 ** j * solution[i])) / 2 ** j for j in range(1, 33)])
        result *= (1 + (i + 1) * t1) ** (10.0 / len(solution) ** 1.2)
    return (result - 1) * 10 / len(solution) ** 2


def happy_cat__(solution=None):
    solution = solution.reshape((-1))
    return (abs(sum(solution ** 2) - len(solution))) ** 0.25 + (0.5 * sum(solution ** 2) + sum(solution)) / len(solution) + 0.5


def hgbat__(solution=None):
    solution = solution.reshape((-1))
    return (abs(sum(solution ** 2) ** 2 - sum(solution) ** 2)) ** 0.5 + (0.5 * sum(solution ** 2) + sum(solution)) / len(solution) + 0.5


def rosenbrock__(solution=None):
    solution = solution.reshape((-1))
    result = 0.0
    for i in range(len(solution) - 1):
        result += 100 * (solution[i] ** 2 - solution[i + 1]) ** 2 + (solution[i] - 1) ** 2
    return result


def rastrigin__(solution=None):
    solution = solution.reshape((-1))
    return sum(solution ** 2 - 10 * cos(2 * pi * solution) + 10)


def elliptic__(solution=None):
    solution = solution.reshape((-1))
    result = 0
    for i in range(len(solution)):
        result += (10 ** 6) ** (i/len(solution)) * solution[i] ** 2
    return result


def modified_schwefel__(solution=None):
    solution = solution.reshape((-1))
    z = solution + 4.209687462275036e+002
    result = 418.9829 * len(solution)
    for i in range(0, len(solution)):
        if z[i] > 500:
            result -= (500 - z[i] % 500) * sin(sqrt(abs(500 - z[i] % 500))) - (z[i] - 500) ** 2 / (10000 * len(solution))
        elif z[i] < -500:
            result -= (z[i] % 500 - 500) * sin(sqrt(abs(z[i] % 500 - 500))) - (z[i] + 500) ** 2 / (10000 * len(solution))
        else:
            result -= z[i] * sin(abs(z[i]) ** 0.5)
    return result


def expanded_scaffer__(solution=None):
    solution = solution.reshape((-1))

    def __xy__(x, y):
        return 0.5 + (sin(sqrt(x ** 2 + y ** 2)) ** 2 - 0.5) / (1 + 0.001 * (x ** 2 + y ** 2)) ** 2

    result = __xy__(solution[-1], solution[0])
    for i in range(0, len(solution) - 1):
        result += __xy__(solution[i], solution[i + 1])
    return result


def griewank__(solution=None):
    solution = solution.reshape((-1))
    result = sum(solution ** 2) / 4000
    temp = 1.0
    for i in range(len(solution)):
        temp *= cos(solution[i] / sqrt(i + 1))
    return result - temp + 1


def expanded_griewank__(solution=None):
    solution = solution.reshape((-1))

    def __rosenbrock__(x=None, y=None):
        return 100 * (x ** 2 - y) ** 2 + (x - 1) ** 2

    def __griewank__(z=None):
        return z ** 2 / 4000 - cos(z / sqrt(1)) + 1

    result = __griewank__(__rosenbrock__(solution[-1], solution[0]))
    for i in range(0, len(solution) - 1):
        result += __griewank__(__rosenbrock__(solution[i], solution[i + 1]))
    return result


def different_powers__(solution=None):
    solution = solution.reshape((-1))
    result = 0.0
    for i in range(0, len(solution)):
        result += power(abs(solution[i]), 2 + 4 * i / (len(solution) - 1))
    return sqrt(result)


def schaffers_f7__(solution=None):
    solution = solution.reshape((-1))
    result = 0.0
    for i in range(0, len(solution) - 1):
        result += power(solution[i] ** 2 + solution[i + 1] ** 2, 0.25) * (1 + sin(50 * sqrt(solution[i] ** 2 + solution[i + 1] ** 2) ** 0.2) ** 2)
    return (result / (len(solution) - 1)) ** 2


def osz_func__(solution=None):
    solution = solution.reshape((-1))
    solution_new = deepcopy(solution)
    for idx in range(0, len(solution)):
        if idx == 0 or idx == len(solution) - 1:
            c1 = 5.5
            c2 = 3.1
            x_sign = 1
            x_star = log(abs(solution[idx]))
            if solution[idx] < 0:
                x_sign = -1
            elif solution[idx] > 0:
                c1 = 10
                c2 = 7.9
            else:
                x_sign = 0
                x_star = 0
            solution_new[idx] = x_sign * exp(x_star + 0.049 * (sin(c1 * x_star) + sin(c2 * x_star)))
    return solution_new


def asy_func__(solution=None, beta=0.5):
    solution = solution.reshape((-1))
    solution_new = deepcopy(solution)
    for idx in range(len(solution)):
        if solution[idx] > 0:
            solution_new[idx] = power(solution[idx], 1 + beta * idx * sqrt(solution[idx]) / (len(solution) - 1))
    return solution_new


def shift__(solution, shift_number=1):
    return solution - shift_number


def rotate__(solution, rotate_number=1):
    return rotate_number*solution


def create_diagonal_matrix__(size=None, alpha=10):
    matrix = zeros((size, size), float)
    temp = [power(alpha, i / (2 * (size - 1))) for i in range(0, size)]
    fill_diagonal(matrix, temp)
    return matrix



#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 22:16, 21/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from numpy import dot, ones, array, ceil
from opfunu.cec.cec2014.utils import *

SUPPORT_DIMENSION = [2, 10, 20, 30, 50, 100]
SUPPORT_DIMENSION_2 = [10, 20, 30, 50, 100]

def F1(solution=None, name="Rotated High Conditioned Elliptic Function", shift_data_file="shift_data_1.txt", bias=100):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_1_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = dot(solution - shift_data, matrix)
    return f1_elliptic__(z) + bias


def F2(solution=None, name="Rotated Bent Cigar Function", shift_data_file="shift_data_2.txt", bias=200):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_2_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = dot(solution - shift_data, matrix)
    return f2_bent_cigar__(z) + bias


def F3(solution=None, name="Rotated Discus Function", shift_data_file="shift_data_3.txt", bias=300):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_3_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = dot(solution - shift_data, matrix)
    return f3_discus__(z) + bias


def F4(solution=None, name="Shifted and Rotated Rosenbrock’s Function", shift_data_file="shift_data_4.txt", bias=400):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_4_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 2.048 * (solution - shift_data) / 100
    z = dot(z, matrix) + 1
    return f4_rosenbrock__(z) + bias


def F5(solution=None, name="Shifted and Rotated Ackley’s Function", shift_data_file="shift_data_5.txt", bias=500):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_5_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = dot(solution - shift_data, matrix)
    return f5_ackley__(z) + bias


def F6(solution=None, name="Shifted and Rotated Weierstrass Function", shift_data_file="shift_data_6.txt", bias=600):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_6_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 0.5 * (solution - shift_data) / 100
    z = dot(z, matrix)
    return f6_weierstrass__(z) + bias


def F7(solution=None, name="Shifted and Rotated Griewank’s Function", shift_data_file="shift_data_7.txt", bias=700):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_7_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 600 * (solution - shift_data) / 100
    z = dot(z, matrix)
    return f7_griewank__(z) + bias


def F8(solution=None, name="Shifted Rastrigin’s Function", shift_data_file="shift_data_8.txt", bias=800):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_8_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 5.12 * (solution - shift_data) / 100
    z = dot(z, matrix)
    return f8_rastrigin__(z) + bias


def F9(solution=None, name="Shifted and Rotated Rastrigin’s Function", shift_data_file="shift_data_9.txt", bias=900):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_9_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 5.12 * (solution - shift_data) / 100
    z = dot(z, matrix)
    return f9_modified_schwefel__(z) + bias


def F10(solution=None, name="Shifted Schwefel’s Function", shift_data_file="shift_data_10.txt", bias=1000):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_10_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 1000 * (solution - shift_data) / 100
    z = dot(z, matrix)
    return f9_modified_schwefel__(z) + bias


def F11(solution=None, name="Shifted and Rotated Schwefel’s Function", shift_data_file="shift_data_11.txt", bias=1100):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_11_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 1000 * (solution - shift_data) / 100
    z = dot(z, matrix)
    return f9_modified_schwefel__(z) + bias


def F12(solution=None, name="Shifted and Rotated Katsuura Function", shift_data_file="shift_data_12.txt", bias=1200):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_12_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 5 * (solution - shift_data) / 100
    z = dot(z, matrix)
    return f10_katsuura__(z) + bias


def F13(solution=None, name="Shifted and Rotated HappyCat Function", shift_data_file="shift_data_13.txt", bias=1300):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_13_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 5 * (solution - shift_data) / 100
    z = dot(z, matrix)
    return f11_happy_cat__(z) + bias


def F14(solution=None, name="Shifted and Rotated HGBat Function", shift_data_file="shift_data_14.txt", bias=1400):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_14_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 5 * (solution - shift_data) / 100
    z = dot(z, matrix)
    return f12_hgbat__(z) + bias


def F15(solution=None, name="Shifted and Rotated Expanded Griewank’s plus Rosenbrock’s Function", shift_data_file="shift_data_15.txt", bias=1500):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_15_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = 5 * (solution - shift_data) / 100
    z = dot(z, matrix) + 1
    return f13_expanded_griewank__(z) + bias


def F16(solution=None, name="Shifted and Rotated Expanded Scaffer’s F6 Function", shift_data_file="shift_data_16.txt", bias=1600):
    problem_size = len(solution)
    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = "M_16_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 2, 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    z = dot(solution - shift_data, matrix) + 1
    return f14_expanded_scaffer__(z) + bias


### ================== Hybrid function ========================

def F17(solution=None, name="Hybrid Function 1", shift_data_file="shift_data_17.txt", bias=1700, shuffle=None):
    problem_size = len(solution)
    p = array([0.3, 0.3, 0.4])
    n1 = int(ceil(p[0] * problem_size))
    n2 = int(ceil(p[1] * problem_size))

    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = "M_17_D" + str(problem_size) + ".txt"
        if shuffle is None:
            f_shuffle = "shuffle_data_17_D" + str(problem_size) + ".txt"
        else:
            f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    shuffle = (load_shift_data__(f_shuffle)[:problem_size] - ones(problem_size)).astype(int)
    idx1 = shuffle[:n1]
    idx2 = shuffle[n1:(n1+n2)]
    idx3 = shuffle[(n1+n2):]
    mz = dot(solution - shift_data, matrix)
    return f9_modified_schwefel__(mz[idx1]) + f8_rastrigin__(mz[idx2]) + f1_elliptic__(mz[idx3]) + bias


def F18(solution=None, name="Hybrid Function 2", shift_data_file="shift_data_18.txt", bias=1800, shuffle=None):
    problem_size = len(solution)
    p = array([0.3, 0.3, 0.4])
    n1 = int(ceil(p[0] * problem_size))
    n2 = int(ceil(p[1] * problem_size))

    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = "M_18_D" + str(problem_size) + ".txt"
        if shuffle is None:
            f_shuffle = "shuffle_data_18_D" + str(problem_size) + ".txt"
        else:
            f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    shuffle = (load_shift_data__(f_shuffle)[:problem_size] - ones(problem_size)).astype(int)
    idx1 = shuffle[:n1]
    idx2 = shuffle[n1:(n1 + n2)]
    idx3 = shuffle[(n1 + n2):]
    mz = dot(solution - shift_data, matrix)
    return f2_bent_cigar__(mz[idx1]) + f12_hgbat__(mz[idx2]) + f8_rastrigin__(mz[idx3]) + bias


def F19(solution=None, name="Hybrid Function 3", shift_data_file="shift_data_19.txt", bias=1900, shuffle=None):
    problem_size = len(solution)
    p = array([0.2, 0.2, 0.3, 0.3])
    n1 = int(ceil(p[0] * problem_size))
    n2 = int(ceil(p[1] * problem_size))
    n3 = int(ceil(p[2] * problem_size))

    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = "M_19_D" + str(problem_size) + ".txt"
        if shuffle is None:
            f_shuffle = "shuffle_data_19_D" + str(problem_size) + ".txt"
        else:
            f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    shuffle = (load_shift_data__(f_shuffle)[:problem_size] - ones(problem_size)).astype(int)
    idx1 = shuffle[:n1]
    idx2 = shuffle[n1:(n1 + n2)]
    idx3 = shuffle[(n1 + n2):(n1+n2+n3)]
    idx4 = shuffle[n1+n2+n3:]
    mz = dot(solution - shift_data, matrix)
    return f7_griewank__(mz[idx1]) + f6_weierstrass__(mz[idx2]) + f4_rosenbrock__(mz[idx3]) + f14_expanded_scaffer__(mz[idx4])+ bias


def F20(solution=None, name="Hybrid Function 4", shift_data_file="shift_data_20.txt", bias=2000, shuffle=None):
    problem_size = len(solution)
    p = array([0.2, 0.2, 0.3, 0.3])
    n1 = int(ceil(p[0] * problem_size))
    n2 = int(ceil(p[1] * problem_size))
    n3 = int(ceil(p[2] * problem_size))

    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = "M_20_D" + str(problem_size) + ".txt"
        if shuffle is None:
            f_shuffle = "shuffle_data_20_D" + str(problem_size) + ".txt"
        else:
            f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    shuffle = (load_shift_data__(f_shuffle)[:problem_size] - ones(problem_size)).astype(int)
    idx1 = shuffle[:n1]
    idx2 = shuffle[n1:(n1 + n2)]
    idx3 = shuffle[(n1 + n2):(n1 + n2 + n3)]
    idx4 = shuffle[n1 + n2 + n3:]
    mz = dot(solution - shift_data, matrix)
    return f12_hgbat__(mz[idx1]) + f3_discus__(mz[idx2]) + f13_expanded_griewank__(mz[idx3]) + f8_rastrigin__(mz[idx4]) + bias


def F21(solution=None, name="Hybrid Function 5", shift_data_file="shift_data_21.txt", bias=2100, shuffle=None):
    problem_size = len(solution)
    p = array([0.1, 0.2, 0.2, 0.2, 0.3])
    n1 = int(ceil(p[0] * problem_size))
    n2 = int(ceil(p[1] * problem_size))
    n3 = int(ceil(p[2] * problem_size))
    n4 = int(ceil(p[3] * problem_size))

    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = "M_21_D" + str(problem_size) + ".txt"
        if shuffle is None:
            f_shuffle = "shuffle_data_21_D" + str(problem_size) + ".txt"
        else:
            f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    shuffle = (load_shift_data__(f_shuffle)[:problem_size] - ones(problem_size)).astype(int)
    idx1 = shuffle[:n1]
    idx2 = shuffle[n1:(n1 + n2)]
    idx3 = shuffle[(n1 + n2):(n1 + n2 + n3)]
    idx4 = shuffle[(n1+n2+n3):(n1+n2+n3+n4)]
    idx5 = shuffle[n1+n2+n3+n4:]
    mz = dot(solution - shift_data, matrix)
    return f14_expanded_scaffer__(mz[idx1]) + f12_hgbat__(mz[idx2]) + f4_rosenbrock__(mz[idx3]) + \
           f9_modified_schwefel__(mz[idx4]) + f1_elliptic__(mz[idx5]) + bias


def F22(solution=None, name="Hybrid Function 6", shift_data_file="shift_data_22.txt", bias=2200, shuffle=None):
    problem_size = len(solution)
    p = array([0.1, 0.2, 0.2, 0.2, 0.3])
    n1 = int(ceil(p[0] * problem_size))
    n2 = int(ceil(p[1] * problem_size))
    n3 = int(ceil(p[2] * problem_size))
    n4 = int(ceil(p[3] * problem_size))

    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = "M_22_D" + str(problem_size) + ".txt"
        if shuffle is None:
            f_shuffle = "shuffle_data_21_D" + str(problem_size) + ".txt"
        else:
            f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_shift_data__(shift_data_file)[:problem_size]
    matrix = load_matrix_data__(f_matrix)
    shuffle = (load_shift_data__(f_shuffle)[:problem_size] - ones(problem_size)).astype(int)
    idx1 = shuffle[:n1]
    idx2 = shuffle[n1:(n1 + n2)]
    idx3 = shuffle[(n1 + n2):(n1 + n2 + n3)]
    idx4 = shuffle[(n1 + n2 + n3):(n1 + n2 + n3 + n4)]
    idx5 = shuffle[n1 + n2 + n3 + n4:]
    mz = dot(solution - shift_data, matrix)
    return f10_katsuura__(mz[idx1]) + f11_happy_cat__(mz[idx2]) + f13_expanded_griewank__(mz[idx3]) + \
           f9_modified_schwefel__(mz[idx4]) + f5_ackley__(mz[idx5]) + bias

### ================== Composition function ========================

def F23(solution=None, name="Composition Function 1", f_shift_file="shift_data_23.txt", f_matrix="M_23_D", f_bias=2300):
    problem_size = len(solution)
    xichma = array([10, 20, 30, 40, 50])
    lamda = array([1, 1e-6, 1e-26, 1e-6, 1e-6])
    bias = array([0, 100, 200, 300, 400])

    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = f_matrix + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_matrix_data__(f_shift_file)[:problem_size]
    shift_data = shift_data[:, :problem_size]
    matrix = load_matrix_data__(f_matrix)

    # 1. Rotated Rosenbrock’s Function F4’
    t1 = solution - shift_data[0]
    g1 = lamda[0] * f4_rosenbrock__(dot(t1, matrix[:problem_size, :])) + bias[0]
    w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * problem_size * xichma[0] ** 2))

    # 2. High Conditioned Elliptic Function F1’
    t2 = solution - shift_data[1]
    g2 = lamda[1] * f1_elliptic__(solution) + bias[1]
    w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * problem_size * xichma[1] ** 2))

    # 3. Rotated Bent Cigar Function F2’
    t3 = solution - shift_data[2]
    g3 = lamda[2] * f2_bent_cigar__(dot(matrix[2 * problem_size: 3 * problem_size, :], t3)) + bias[2]
    w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * problem_size * xichma[2] ** 2))

    # 4. Rotated Discus Function F3’
    t4 = solution - shift_data[3]
    g4 = lamda[3] * f3_discus__(dot(matrix[3 * problem_size: 4 * problem_size, :], t4)) + bias[3]
    w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * problem_size * xichma[3] ** 2))

    # 4. High Conditioned Elliptic Function F1’
    t5 = solution - shift_data[4]
    g5 = lamda[4] * f1_elliptic__(solution) + bias[4]
    w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * problem_size * xichma[4] ** 2))

    sw = sum([w1, w2, w3, w4, w5])
    result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
    return result + f_bias


def F24(solution=None, name="Composition Function 2", f_shift_file="shift_data_24.txt", f_matrix="M_24_D", f_bias=2400):
    problem_size = len(solution)
    xichma = array([20, 20, 20])
    lamda = array([1, 1, 1])
    bias = array([0, 100, 200])

    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = f_matrix + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_matrix_data__(f_shift_file)[:problem_size]
    shift_data = shift_data[:, :problem_size]
    matrix = load_matrix_data__(f_matrix)

    # 1. Rotated Rosenbrock’s Function F4’
    t1 = solution - shift_data[0]
    g1 = lamda[0] * f9_modified_schwefel__(solution) + bias[0]
    w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * problem_size * xichma[0] ** 2))

    # 2. Rotated Rastrigin’s Function F9’
    t2 = solution - shift_data[1]
    g2 = lamda[1] * f8_rastrigin__(dot(matrix[problem_size: 2 * problem_size], t2)) + bias[1]
    w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * problem_size * xichma[1] ** 2))

    # 3. Rotated HGBat Function F14’
    t3 = solution - shift_data[2]
    g3 = lamda[2] * f12_hgbat__(dot(matrix[2 * problem_size: 3 * problem_size, :], t3)) + bias[2]
    w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * problem_size * xichma[2] ** 2))

    sw = sum([w1, w2, w3])
    result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
    return result + f_bias


def F25(solution=None, name="Composition Function 3", f_shift_file="shift_data_25.txt", f_matrix="M_25_D", f_bias=2500):
    problem_size = len(solution)
    xichma = array([10, 30, 50])
    lamda = array([0.25, 1, 1e-7])
    bias = array([0, 100, 200])

    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = f_matrix + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_matrix_data__(f_shift_file)[:problem_size]
    shift_data = shift_data[:, :problem_size]
    matrix = load_matrix_data__(f_matrix)

    # 1. Rotated Schwefel's Function F11’
    t1 = solution - shift_data[0]
    g1 = lamda[0] * f9_modified_schwefel__(dot(matrix[:problem_size, :problem_size], t1)) + bias[0]
    w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * problem_size * xichma[0] ** 2))

    # 2. Rotated Rastrigin’s Function F9’
    t2 = solution - shift_data[1]
    g2 = lamda[1] * f8_rastrigin__(dot(matrix[problem_size: 2 * problem_size], t2)) + bias[1]
    w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * problem_size * xichma[1] ** 2))

    # 3. Rotated High Conditioned Elliptic Function F1'
    t3 = solution - shift_data[2]
    g3 = lamda[2] * f1_elliptic__(dot(matrix[2 * problem_size: 3 * problem_size, :], t3)) + bias[2]
    w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * problem_size * xichma[2] ** 2))

    sw = sum([w1, w2, w3])
    result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
    return result + f_bias


def F26(solution=None, name="Composition Function 4", f_shift_file="shift_data_26.txt", f_matrix="M_26_D", f_bias=2600):
    problem_size = len(solution)
    xichma = array([10, 10, 10, 10, 10])
    lamda = array([0.25, 1, 1e-7, 2.5, 10])
    bias = array([0, 100, 200, 300, 400])

    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = f_matrix + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_matrix_data__(f_shift_file)[:problem_size]
    shift_data = shift_data[:, :problem_size]
    matrix = load_matrix_data__(f_matrix)

    # 1. Rotated Schwefel's Function F11’
    t1 = solution - shift_data[0]
    g1 = lamda[0] * f9_modified_schwefel__(dot(matrix[:problem_size, :], t1)) + bias[0]
    w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * problem_size * xichma[0] ** 2))

    # 2. Rotated HappyCat Function F13’
    t2 = solution - shift_data[1]
    g2 = lamda[1] * f11_happy_cat__(dot(matrix[problem_size:2 * problem_size, :], t2)) + bias[1]
    w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * problem_size * xichma[1] ** 2))

    # 3. Rotated High Conditioned Elliptic Function F1’
    t3 = solution - shift_data[2]
    g3 = lamda[2] * f1_elliptic__(dot(matrix[2 * problem_size: 3 * problem_size, :], t3)) + bias[2]
    w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * problem_size * xichma[2] ** 2))

    # 4. Rotated Weierstrass Function F6’
    t4 = solution - shift_data[3]
    g4 = lamda[3] * f6_weierstrass__(dot(matrix[3 * problem_size: 4 * problem_size, :], t4)) + bias[3]
    w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * problem_size * xichma[3] ** 2))

    # 5. Rotated Griewank’s Function F7’
    t5 = solution - shift_data[4]
    g5 = lamda[4] * f7_griewank__(dot(matrix[4*problem_size:, :], t5)) + bias[4]
    w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * problem_size * xichma[4] ** 2))

    sw = sum([w1, w2, w3, w4, w5])
    result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
    return result + f_bias


def F27(solution=None, name="Composition Function 5", f_shift_file="shift_data_27.txt", f_matrix="M_27_D", f_bias=2700):
    problem_size = len(solution)
    xichma = array([10, 10, 10, 20, 20])
    lamda = array([10, 10, 2.5, 25, 1e-6])
    bias = array([0, 100, 200, 300, 400])

    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = f_matrix + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_matrix_data__(f_shift_file)[:problem_size]
    shift_data = shift_data[:, :problem_size]
    matrix = load_matrix_data__(f_matrix)

    # 1. Rotated HGBat Function F14'
    t1 = solution - shift_data[0]
    g1 = lamda[0] * f12_hgbat__(dot(matrix[:problem_size, :], t1)) + bias[0]
    w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * problem_size * xichma[0] ** 2))

    # 2. Rotated Rastrigin’s Function F9’
    t2 = solution - shift_data[1]
    g2 = lamda[1] * f8_rastrigin__(dot(matrix[problem_size:2 * problem_size, :], t2)) + bias[1]
    w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * problem_size * xichma[1] ** 2))

    # 3. Rotated Schwefel's Function F11’
    t3 = solution - shift_data[2]
    g3 = lamda[2] * f9_modified_schwefel__(dot(matrix[2 * problem_size: 3 * problem_size, :], t3)) + bias[2]
    w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * problem_size * xichma[2] ** 2))

    # 4. Rotated Weierstrass Function F6’
    t4 = solution - shift_data[3]
    g4 = lamda[3] * f6_weierstrass__(dot(matrix[3 * problem_size: 4 * problem_size, :], t4)) + bias[3]
    w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * problem_size * xichma[3] ** 2))

    # 5. Rotated High Conditioned Elliptic Function F1’
    t5 = solution - shift_data[4]
    g5 = lamda[4] * f1_elliptic__(dot(matrix[4 * problem_size:, :], t5)) + bias[4]
    w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * problem_size * xichma[4] ** 2))

    sw = sum([w1, w2, w3, w4, w5])
    result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
    return result + f_bias


def F28(solution=None, name="Composition Function 6", f_shift_file="shift_data_28.txt", f_matrix="M_28_D", f_bias=2800):
    problem_size = len(solution)
    xichma = array([10, 20, 30, 40, 50])
    lamda = array([2.5, 10, 2.5, 5e-4, 1e-6])
    bias = array([0, 100, 200, 300, 400])

    if problem_size in SUPPORT_DIMENSION_2:
        f_matrix = f_matrix + str(problem_size) + ".txt"
    else:
        print("CEC 2014 function only support problem size 10, 20, 30, 50, 100")
        return 1
    shift_data = load_matrix_data__(f_shift_file)[:problem_size]
    shift_data = shift_data[:, :problem_size]
    matrix = load_matrix_data__(f_matrix)

    # 1. Rotated Expanded Griewank’s plus Rosenbrock’s Function F15’
    t1 = solution - shift_data[0]
    g1 = lamda[0] * F15(solution, bias=0) + bias[0]
    w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * problem_size * xichma[0] ** 2))

    # 2. Rotated HappyCat Function F13’
    t2 = solution - shift_data[1]
    g2 = lamda[1] * f11_happy_cat__(dot(matrix[problem_size:2 * problem_size, :], t2)) + bias[1]
    w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * problem_size * xichma[1] ** 2))

    # 3. Rotated Schwefel's Function F11’
    t3 = solution - shift_data[2]
    g3 = lamda[2] * f9_modified_schwefel__(dot(matrix[2 * problem_size: 3 * problem_size, :], t3)) + bias[2]
    w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * problem_size * xichma[2] ** 2))

    # 4. Rotated Expanded Scaffer’s F6 Function F16’
    t4 = solution - shift_data[3]
    g4 = lamda[3] * f14_expanded_scaffer__(dot(matrix[3 * problem_size: 4 * problem_size, :], t4)) + bias[3]
    w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * problem_size * xichma[3] ** 2))

    # 5. Rotated High Conditioned Elliptic Function F1’
    t5 = solution - shift_data[4]
    g5 = lamda[4] * f1_elliptic__(dot(matrix[4 * problem_size:, :], t5)) + bias[4]
    w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * problem_size * xichma[4] ** 2))

    sw = sum([w1, w2, w3, w4, w5])
    result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
    return result + f_bias


def F29(solution=None, name="Composition Function 7", shift_data_file="shift_data_29.txt", f_bias=2900):
    num_funcs = 3
    problem_size = len(solution)
    xichma = array([10, 30, 50])
    lamda = array([1, 1, 1])
    bias = array([0, 100, 200])

    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    shift_data = load_matrix_data__(shift_data_file)[:problem_size]
    shift_data = shift_data[:, :problem_size]

    def __fi__(solution=None, idx=None):
        if idx == 0:
            return F17(solution, bias=0, shuffle=29)
        elif idx == 1:
            return F18(solution, bias=0, shuffle=29)
        else:
            return F19(solution, bias=0, shuffle=29)

    weights = ones(num_funcs)
    fits = ones(num_funcs)
    for i in range(0, num_funcs):
        t1 = lamda[i] * __fi__(solution, i) + bias[i]
        t2 = 1.0 / sqrt(sum((solution - shift_data[i]) ** 2))
        w_i = t2 * exp(-sum((solution - shift_data[i]) ** 2) / (2 * problem_size * xichma[i] ** 2))
        weights[i] = w_i
        fits[i] = t1
    sw = sum(weights)
    result = 0.0
    for i in range(0, num_funcs):
        result += (weights[i] / sw) * fits[i]
    return result + f_bias


def F30(solution=None, name="Composition Function 8", shift_data_file="shift_data_30.txt", f_bias=3000):
    num_funcs = 3
    problem_size = len(solution)
    xichma = array([10, 30, 50])
    lamda = array([1, 1, 1])
    bias = array([0, 100, 200])

    if problem_size > 100:
        print("CEC 2014 not support for problem size > 100")
        return 1
    shift_data = load_matrix_data__(shift_data_file)[:problem_size]
    shift_data = shift_data[:, :problem_size]

    def __fi__(solution=None, idx=None):
        if idx == 0:
            return F20(solution, bias=0, shuffle=30)
        elif idx == 1:
            return F21(solution, bias=0, shuffle=30)
        else:
            return F22(solution, bias=0, shuffle=30)

    weights = ones(num_funcs)
    fits = ones(num_funcs)
    for i in range(0, num_funcs):
        t1 = lamda[i] * __fi__(solution, i) + bias[i]
        t2 = 1.0 / sqrt(sum((solution - shift_data[i]) ** 2))
        w_i = t2 * exp(-sum((solution - shift_data[i]) ** 2) / (2 * problem_size * xichma[i] ** 2))
        weights[i] = w_i
        fits[i] = t1
    sw = sum(weights)
    result = 0.0
    for i in range(0, num_funcs):
        result += (weights[i] / sw) * fits[i]
    return result + f_bias





#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 14:26, 26/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
# -------------------------------------------------------------------------------------------------------%

from numpy import array, ceil, concatenate, arange, ones, zeros
from opfunu.cec_basic.utils import *


def F1(solution=None, shift_num=1, rotate_num=1, f_bias=100):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return elliptic__(z) + f_bias


def F2(solution=None, shift_num=1, rotate_num=1, f_bias=200):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return bent_cigar__(z) + f_bias


def F3(solution=None, shift_num=1, rotate_num=1, f_bias=300):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return discus__(z) + f_bias


def F4(solution=None, shift_num=1, rotate_num=2.048/100, f_bias=400):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num) + 1
    return rosenbrock__(z) + f_bias


def F5(solution=None, shift_num=1, rotate_num=1, f_bias=500):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return ackley__(z) + f_bias


def F6(solution=None, shift_num=1,  rotate_num=0.5 / 100, f_bias=600):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return weierstrass__(z) + f_bias


def F7(solution=None, shift_num=1, rotate_num=600/100, f_bias=700):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return griewank__(z) + f_bias


def F8(solution=None, shift_num=1, rotate_num=5.12 / 100, f_bias=800):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return rastrigin__(z) + f_bias


def F9(solution=None, shift_num=1, rotate_num=5.12 / 100, f_bias=900):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return rastrigin__(z) + f_bias


def F10(solution=None, shift_num=1, rotate_num=1000 / 100, f_bias=1000):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return modified_schwefel__(z) + f_bias


def F11(solution=None, shift_num=1, rotate_num=1000 / 100, f_bias=1100):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return modified_schwefel__(z) + f_bias


def F12(solution=None, shift_num=1, rotate_num=5 / 100, f_bias=1200):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return katsuura__(z) + f_bias


def F13(solution=None, shift_num=1, rotate_num=5 / 100, f_bias=1300):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return katsuura__(z) + f_bias


def F14(solution=None, shift_num=1, rotate_num=5 / 100, f_bias=1400):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return hgbat__(z) + f_bias


def F15(solution=None, shift_num=1, rotate_num=5 / 100, f_bias=1500):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num) + 1
    return expanded_griewank__(z) + f_bias


def F16(solution=None, shift_num=1, rotate_num=1, f_bias=1600):
    x = shift__(solution, shift_num)
    z = rotate__(x, rotate_num)
    return expanded_scaffer__(z) + f_bias


def F17(solution=None, shift_num=1, rotate_num=1, f_bias=1700):
    dim = len(solution)
    n1 = int(ceil(0.3 * dim))
    n2 = int(ceil(0.3 * dim)) + n1
    D = arange(dim)
    x = shift__(solution, shift_num)
    return modified_schwefel__(x[D[:n1]]) + rastrigin__(x[D[n1:n2]]) + elliptic__(x[D[n2:]]) + f_bias


def F18(solution=None, shift_num=1, rotate_num=1, f_bias=1800):
    dim = len(solution)
    n1 = int(ceil(0.3 * dim))
    n2 = int(ceil(0.3 * dim)) + n1
    D = arange(dim)
    x = shift__(solution, shift_num)
    return bent_cigar__(x[D[:n1]]) + hgbat__(x[D[n1:n2]]) + rastrigin__(x[D[n2:]]) + f_bias


def F19(solution=None, shift_num=1, rotate_num=1, f_bias=1900):
    dim = len(solution)
    n1 = int(ceil(0.2 * dim))
    n2 = int(ceil(0.2 * dim)) + n1
    n3 = int(ceil(0.3 * dim)) + n2
    D = arange(dim)
    x = shift__(solution, shift_num)
    return griewank__(x[D[:n1]]) + weierstrass__(x[D[n1:n2]]) + rosenbrock__(x[D[n2:n3]]) + expanded_scaffer__(x[D[n3:]]) + f_bias


def F20(solution=None, shift_num=1, rotate_num=1, f_bias=2000):
    dim = len(solution)
    n1 = int(ceil(0.2 * dim))
    n2 = int(ceil(0.2 * dim)) + n1
    n3 = int(ceil(0.3 * dim)) + n2
    D = arange(dim)
    x = shift__(solution, shift_num)
    return hgbat__(x[D[:n1]]) + discus__(x[D[n1:n2]]) + expanded_griewank__(x[D[n2:n3]]) + rastrigin__(x[D[n3:]]) + f_bias


def F21(solution=None, shift_num=1, rotate_num=1, f_bias=2100):
    dim = len(solution)
    n1 = int(ceil(0.1 * dim))
    n2 = int(ceil(0.2 * dim)) + n1
    n3 = int(ceil(0.2 * dim)) + n2
    n4 = int(ceil(0.2 * dim)) + n3
    D = arange(dim)
    x = shift__(solution, shift_num)
    return expanded_scaffer__(x[D[:n1]]) + hgbat__(x[D[n1:n2]]) + rosenbrock__(x[D[n2:n3]]) + \
           modified_schwefel__(x[D[n3:n4]]) + elliptic__(x[D[n4:]]) + f_bias


def F22(solution=None, shift_num=1, rotate_num=1, f_bias=2200):
    dim = len(solution)
    n1 = int(ceil(0.1 * dim))
    n2 = int(ceil(0.2 * dim)) + n1
    n3 = int(ceil(0.2 * dim)) + n2
    n4 = int(ceil(0.2 * dim)) + n3
    D = arange(dim)
    x = shift__(solution, shift_num)
    return katsuura__(x[D[:n1]]) + happy_cat__(x[D[n1:n2]]) + expanded_griewank__(x[D[n2:n3]]) + \
           modified_schwefel__(x[D[n3:n4]]) + ackley__(x[D[n4:]]) + f_bias


def F23(solution=None, shift_num=1, rotate_num=1, f_bias=2300):
    shift_arr = [1, 2, 3, 4, 5]
    sigma = [10, 20, 30, 40, 50]
    lamda = [1, 1.0e-6, 1.0e-26, 1.0e-6, 1.0e-6]
    bias = [0, 100, 200, 300, 400]
    fun = [F4, F1, F2, F3, F1]
    dim = len(solution)
    result = 0
    w = ones(len(shift_arr))
    for i in range(len(shift_arr)):
        x = shift__(solution, shift_arr[i])
        w[i] = 1 / sqrt(sum(x**2)) * exp(- sum(x**2) / (2 * dim * sigma[i]**2))
    sumw = sum(w)
    for i in range(len(shift_arr)):
        result += (w[i] / sumw) * (lamda[i] * fun[i](solution, shift_num, rotate_num, f_bias=0) + bias[i])
    return result + f_bias


def F24(solution=None, shift_num=1, rotate_num=1, f_bias=2400):
    shift_arr = [1, 2, 3]
    sigma = [20, 20, 20]
    lamda = [1, 1, 1]
    bias = [0, 100, 200]
    fun = [F10, F9, F14]
    dim = len(solution)
    result = 0
    w = ones(len(shift_arr))
    for i in range(len(shift_arr)):
        x = shift__(solution, shift_arr[i])
        w[i] = 1 / sqrt(sum(x ** 2)) * exp(- sum(x ** 2) / (2 * dim * sigma[i] ** 2))
    sumw = sum(w)
    for i in range(len(shift_arr)):
        result += (w[i] / sumw) * (lamda[i] * fun[i](solution, shift_num, rotate_num, f_bias=0) + bias[i])
    return result + f_bias


def F25(solution=None, shift_num=1, rotate_num=1, f_bias=2500):
    shift_arr = [1, 2, 3]
    sigma = [10, 30, 50]
    lamda = [0.25, 1, 1.0e-7]
    bias = [0, 100, 200]
    fun = [F11, F9, F1]
    dim = len(solution)
    result = 0
    w = ones(len(shift_arr))
    for i in range(len(shift_arr)):
        x = shift__(solution, shift_arr[i])
        w[i] = 1 / sqrt(sum(x ** 2)) * exp(- sum(x ** 2) / (2 * dim * sigma[i] ** 2))
    sumw = sum(w)
    for i in range(len(shift_arr)):
        result += (w[i] / sumw) * (lamda[i] * fun[i](solution, shift_num, rotate_num, f_bias=0) + bias[i])
    return result + f_bias


def F26(solution=None, shift_num=1, rotate_num=1, f_bias=2600):
    shift_arr = [1, 2, 3, 4, 5]
    sigma = [10, 10, 10, 10, 10]
    lamda = [0.25, 1.0, 1.0e-7, 2.5, 10.0]
    bias = [0, 100, 200, 300, 400]
    fun = [F11, F13, F1, F6, F7]
    dim = len(solution)
    result = 0
    w = ones(len(shift_arr))
    for i in range(len(shift_arr)):
        x = shift__(solution, shift_arr[i])
        w[i] = 1 / sqrt(sum(x ** 2)) * exp(- sum(x ** 2) / (2 * dim * sigma[i] ** 2))
    sumw = sum(w)
    for i in range(len(shift_arr)):
        result += (w[i] / sumw) * (lamda[i] * fun[i](solution, shift_num, rotate_num, f_bias=0) + bias[i])
    return result + f_bias


def F27(solution=None, shift_num=1, rotate_num=1, f_bias=2700):
    shift_arr = [1, 2, 3, 4, 5]
    sigma = [10, 10, 10, 20, 20]
    lamda = [10, 10, 2.5, 25, 1.0e-6]
    bias = [0, 100, 200, 300, 400]
    fun = [F14, F9, F11, F6, F1]
    dim = len(solution)
    result = 0
    w = ones(len(shift_arr))
    for i in range(len(shift_arr)):
        x = shift__(solution, shift_arr[i])
        w[i] = 1 / sqrt(sum(x ** 2)) * exp(- sum(x ** 2) / (2 * dim * sigma[i] ** 2))
    sumw = sum(w)
    for i in range(len(shift_arr)):
        result += (w[i] / sumw) * (lamda[i] * fun[i](solution, shift_num, rotate_num, f_bias=0) + bias[i])
    return result + f_bias


def F28(solution=None, shift_num=1, rotate_num=1, f_bias=2800):
    shift_arr = [1, 2, 3, 4, 5]
    sigma = [10, 20, 30, 40, 50]
    lamda = [2.5, 10, 2.5, 5.0e-4, 1.0e-6]
    bias = [0, 100, 200, 300, 400]
    fun = [F15, F13, F11, F16, F1]
    dim = len(solution)
    result = 0
    w = ones(len(shift_arr))
    for i in range(len(shift_arr)):
        x = shift__(solution, shift_arr[i])
        w[i] = 1 / sqrt(sum(x ** 2)) * exp(- sum(x ** 2) / (2 * dim * sigma[i] ** 2))
    sumw = sum(w)
    for i in range(len(shift_arr)):
        result += (w[i] / sumw) * (lamda[i] * fun[i](solution, shift_num, rotate_num, f_bias=0) + bias[i])
    return result + f_bias


def F29(solution=None, shift_num=1, rotate_num=1, f_bias=2900):
    shift_arr = [4, 5, 6]
    sigma = [10, 30, 50]
    lamda = [1, 1, 1]
    bias = [0, 100, 200]
    fun = [F17, F18, F19]
    dim = len(solution)
    result = 0
    w = ones(len(shift_arr))
    for i in range(len(shift_arr)):
        x = shift__(solution, shift_arr[i])
        w[i] = 1 / sqrt(sum(x ** 2)) * exp(- sum(x ** 2) / (2 * dim * sigma[i] ** 2))
    sumw = sum(w)
    for i in range(len(shift_arr)):
        result += (w[i] / sumw) * (lamda[i] * fun[i](solution, shift_num, rotate_num, f_bias=0) + bias[i])
    return result + f_bias


def F30(solution=None, shift_num=1, rotate_num=1, f_bias=3000):
    shift_arr = [1, 2, 3]
    sigma = [10, 30, 50]
    lamda = [1, 1, 1]
    bias = [0, 100, 200]
    fun = [F20, F21, F22]
    dim = len(solution)
    result = 0
    w = ones(len(shift_arr))
    for i in range(len(shift_arr)):
        x = shift__(solution, shift_arr[i])
        w[i] = 1 / sqrt(sum(x ** 2)) * exp(- sum(x ** 2) / (2 * dim * sigma[i] ** 2))
    sumw = sum(w)
    for i in range(len(shift_arr)):
        result += (w[i] / sumw) * (lamda[i] * fun[i](solution, shift_num, rotate_num, f_bias=0) + bias[i])
    return result + f_bias




#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 01:13, 26/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
# -------------------------------------------------------------------------------------------------------%

from numpy import dot, array, sum, matmul, where, sqrt, sign, min, cos, pi, exp, round, ceil, ones, concatenate
from opfunu.cec.utils import BasicFunction


class Model(BasicFunction):
    def __init__(self, solution=None, cec_type="cec2014", bound=(-100, 100), dimensions=(10, 20, 30, 50, 100)):
        BasicFunction.__init__(self, cec_type)
        self.problem_size = len(solution)
        self.dimensions = dimensions
        self.check_dimensions(self.problem_size)
        self.bound = bound
        self.solution = solution

    def __load_shift_matrix_1__(self, f_shift=None, f_matrix=None):
        shift = self.load_shift_data__(f_shift)[:self.problem_size]
        matrix = self.load_matrix_data__(f_matrix)
        return shift, matrix

    def __load_shift_matrix_2__(self, f_shift=None, f_matrix=None):
        shift = self.load_matrix_data__(f_shift)[:, :self.problem_size]
        matrix = self.load_matrix_data__(f_matrix)
        return shift, matrix

    def __load_shift_matrix_shuffle__(self, f_shift=None, f_matrix=None, f_shuffle=None):
        shift = self.load_shift_data__(f_shift)[:self.problem_size]
        matrix = self.load_matrix_data__(f_matrix)
        shuffle = (self.load_shift_data__(f_shuffle)[:self.problem_size] - ones(self.problem_size)).astype(int)
        return shift, matrix, shuffle


    def F1(self, name="Rotated High Conditioned Elliptic Function", f_shift="shift_data_1.txt", f_matrix="M_1_D", f_bias=100):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        t1 = self.solution - shift
        z = dot(matrix, t1)
        return self.elliptic__(z) + f_bias

    def F2(self, name="Rotated Bent Cigar Function", f_shift="shift_data_2.txt", f_matrix="M_2_D", f_bias=200):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = dot(matrix, self.solution - shift)
        return self.bent_cigar__(z) + f_bias

    def F3(self, name="Rotated Discus Function", f_shift="shift_data_3.txt", f_matrix="M_3_D", f_bias=300):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = dot(matrix, self.solution - shift)
        return self.discus__(z) + f_bias

    def F4(self, name="Shifted and Rotated Rosenbrock’s Function", f_shift="shift_data_4.txt", f_matrix="M_4_D", f_bias=400):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 2.048 * (self.solution - shift) / 100
        y = dot(matrix, z) + 1
        return self.rosenbrock__(y) + f_bias

    def F5(self, name="Shifted and Rotated Ackley’s Function", f_shift="shift_data_5.txt", f_matrix="M_5_D", f_bias=500):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = dot(matrix, self.solution - shift)
        return self.ackley__(z) + f_bias

    def F6(self, name="Shifted and Rotated Weierstrass Function", f_shift="shift_data_6.txt", f_matrix="M_6_D", f_bias=600):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 0.5 * (self.solution - shift) / 100
        y = dot(matrix, z)
        return self.weierstrass__(y) + f_bias

    def F7(self, name="Shifted and Rotated Griewank’s Function", f_shift="shift_data_7.txt", f_matrix="M_7_D", f_bias=700):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 600 * (self.solution - shift) / 100
        y = dot(matrix, z)
        return self.griewank__(y) + f_bias

    def F8(self, name="Shifted Rastrigin’s Function", f_shift="shift_data_8.txt", f_matrix="M_8_D", f_bias=800):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 5.12 * (self.solution - shift) / 100
        return self.rastrigin__(z) + f_bias

    def F9(self, name="Shifted and Rotated Rastrigin’s Function", f_shift="shift_data_9.txt", f_matrix="M_9_D", f_bias=900):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 5.12 * (self.solution - shift) / 100
        y = dot(matrix, z)
        return self.griewank__(y) + f_bias

    def F10(self, name="Shifted Schwefel’s Function", f_shift="shift_data_10.txt", f_matrix="M_10_D", f_bias=1000):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 1000 * (self.solution - shift) / 100
        return self.modified_schwefel__(z) + f_bias

    def F11(self, name="Shifted and Rotated Schwefel’s Function", f_shift="shift_data_11.txt", f_matrix="M_11_D", f_bias=1100):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 1000 * (self.solution - shift) / 100
        y = dot(matrix, z)
        return self.modified_schwefel__(y) + f_bias

    def F12(self, name="Shifted and Rotated Katsuura Function", f_shift="shift_data_12.txt", f_matrix="M_12_D", f_bias=1200):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 5 * (self.solution - shift) / 100
        y = dot(matrix, z)
        return self.katsuura__(y) + f_bias

    def F13(self, name="Shifted and Rotated HappyCat Function", f_shift="shift_data_13.txt", f_matrix="M_13_D", f_bias=1300):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 5 * (self.solution - shift) / 100
        y = dot(matrix, z)
        return self.happy_cat__(y) + f_bias

    def F14(self, name="Shifted and Rotated HGBat Function", f_shift="shift_data_14.txt", f_matrix="M_14_D", f_bias=1400):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 5 * (self.solution - shift) / 100
        y = dot(matrix, z)
        return self.hgbat__(y) + f_bias

    def F15(self, name="Shifted and Rotated Expanded Griewank’s plus Rosenbrock’s Function",
            f_shift="shift_data_15.txt", f_matrix="M_15_D", f_bias=1500):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = 5 * (self.solution - shift) / 100
        y = dot(matrix, z) + 1
        return self.expanded_griewank__(y) + f_bias

    def F16(self, name="Shifted and Rotated Expanded Scaffer’s F6 Function", f_shift="shift_data_16.txt", f_matrix="M_16_D", f_bias=1600):
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_1__(f_shift, f_matrix)
        z = dot(matrix, self.solution - shift) + 1
        return self.expanded_scaffer__(z) + f_bias

    ### Hybrid functions
    def F17(self, name="Hybrid Function 1", f_shift="shift_data_17.txt", f_matrix="M_17_D", shuffle=17, f_bias=1700):
        p = array([0.3, 0.3, 0.4])
        n1 = int(ceil(p[0] * self.problem_size))
        n2 = int(ceil(p[1] * self.problem_size)) + n1
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(self.problem_size) + ".txt"
        shift, matrix, shuffle = self.__load_shift_matrix_shuffle__(f_shift, f_matrix, f_shuffle)
        idx1 = shuffle[:n1]
        idx2 = shuffle[n1:n2]
        idx3 = shuffle[n2:]
        z = self.solution - shift
        z1 = concatenate((z[idx1], z[idx2], z[idx3]))
        mz = dot(matrix, z1)
        return self.modified_schwefel__(mz[:n1]) + self.rastrigin__(mz[n1:n2]) + self.elliptic__(mz[n2:]) + f_bias

    def F18(self, name="Hybrid Function 2", f_shift="shift_data_18.txt", f_matrix="M_18_D", shuffle=18, f_bias=1800):
        p = array([0.3, 0.3, 0.4])
        n1 = int(ceil(p[0] * self.problem_size))
        n2 = int(ceil(p[1] * self.problem_size)) + n1
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(self.problem_size) + ".txt"
        shift, matrix, shuffle = self.__load_shift_matrix_shuffle__(f_shift, f_matrix, f_shuffle)
        idx1 = shuffle[:n1]
        idx2 = shuffle[n1:n2]
        idx3 = shuffle[n2:]
        z = self.solution - shift
        z1 = concatenate((z[idx1], z[idx2], z[idx3]))
        mz = dot(matrix, z1)
        return self.bent_cigar__(mz[:n1]) + self.hgbat__(mz[n1:n2]) + self.rastrigin__(mz[n2:]) + f_bias

    def F19(self, name="Hybrid Function 3", f_shift="shift_data_19.txt", f_matrix="M_19_D", shuffle=19, f_bias=1900):
        p = array([0.2, 0.2, 0.3, 0.3])
        n1 = int(ceil(p[0] * self.problem_size))
        n2 = int(ceil(p[1] * self.problem_size)) + n1
        n3 = int(ceil(p[2] * self.problem_size)) + n2
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(self.problem_size) + ".txt"
        shift, matrix, shuffle = self.__load_shift_matrix_shuffle__(f_shift, f_matrix, f_shuffle)
        idx1 = shuffle[:n1]
        idx2 = shuffle[n1:n2]
        idx3 = shuffle[n2:n3]
        idx4 = shuffle[n3:]
        z = self.solution - shift
        z1 = concatenate((z[idx1], z[idx2], z[idx3], z[idx4]))
        mz = dot(matrix, z1)
        return self.griewank__(mz[:n1]) + self.weierstrass__(mz[n1:n2]) + self.rosenbrock__(mz[n2:n3]) + self.expanded_scaffer__(mz[n3:]) + f_bias

    def F20(self, name="Hybrid Function 4", f_shift="shift_data_20.txt", f_matrix="M_20_D", shuffle=20, f_bias=2000):
        p = array([0.2, 0.2, 0.3, 0.3])
        n1 = int(ceil(p[0] * self.problem_size))
        n2 = int(ceil(p[1] * self.problem_size)) + n1
        n3 = int(ceil(p[2] * self.problem_size)) + n2
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(self.problem_size) + ".txt"
        shift, matrix, shuffle = self.__load_shift_matrix_shuffle__(f_shift, f_matrix, f_shuffle)
        idx1 = shuffle[:n1]
        idx2 = shuffle[n1:n2]
        idx3 = shuffle[n2:n3]
        idx4 = shuffle[n3:]
        z = self.solution - shift
        z1 = concatenate((z[idx1], z[idx2], z[idx3], z[idx4]))
        mz = dot(matrix, z1)
        return self.hgbat__(mz[:n1]) + self.discus__(mz[n1:n2]) + self.expanded_griewank__(mz[n2:n3]) + self.rastrigin__(mz[n3:]) + f_bias

    def F21(self, name="Hybrid Function 5", f_shift="shift_data_21.txt", f_matrix="M_21_D", shuffle=21, f_bias=2100):
        p = array([0.1, 0.2, 0.2, 0.2, 0.3])
        n1 = int(ceil(p[0] * self.problem_size))
        n2 = int(ceil(p[1] * self.problem_size)) + n1
        n3 = int(ceil(p[2] * self.problem_size)) + n2
        n4 = int(ceil(p[3] * self.problem_size)) + n3
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(self.problem_size) + ".txt"
        shift, matrix, shuffle = self.__load_shift_matrix_shuffle__(f_shift, f_matrix, f_shuffle)
        idx1 = shuffle[:n1]
        idx2 = shuffle[n1:n2]
        idx3 = shuffle[n2:n3]
        idx4 = shuffle[n3:n4]
        idx5 = shuffle[n4:]
        z = self.solution - shift
        z1 = concatenate((z[idx1], z[idx2], z[idx3], z[idx4], z[idx5]))
        mz = dot(matrix, z1)
        return self.expanded_scaffer__(mz[:n1]) + self.hgbat__(mz[n1:n2]) + self.rosenbrock__(mz[n2:n3]) + \
               self.modified_schwefel__(mz[n3:n4]) + self.elliptic__(mz[n4:]) + f_bias

    def F22(self, name="Hybrid Function 6", f_shift="shift_data_22.txt", f_matrix="M_22_D", shuffle=22, f_bias=2200):
        p = array([0.1, 0.2, 0.2, 0.2, 0.3])
        n1 = int(ceil(p[0] * self.problem_size))
        n2 = int(ceil(p[1] * self.problem_size)) + n1
        n3 = int(ceil(p[2] * self.problem_size)) + n2
        n4 = int(ceil(p[3] * self.problem_size)) + n3
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        f_shuffle = "shuffle_data_" + str(shuffle) + "_D" + str(self.problem_size) + ".txt"
        shift, matrix, shuffle = self.__load_shift_matrix_shuffle__(f_shift, f_matrix, f_shuffle)
        idx1 = shuffle[:n1]
        idx2 = shuffle[n1:n2]
        idx3 = shuffle[n2:n3]
        idx4 = shuffle[n3:n4]
        idx5 = shuffle[n4:]
        z = self.solution - shift
        z1 = concatenate((z[idx1], z[idx2], z[idx3], z[idx4], z[idx5]))
        mz = dot(matrix, z1)
        return self.katsuura__(mz[:n1]) + self.happy_cat__(mz[n1:n2]) + self.expanded_griewank__(mz[n2:n3]) + \
               self.modified_schwefel__(mz[n3:n4]) + self.ackley__(mz[n4:]) + f_bias

    ## Composition functions

    def F23(self, name="Composition Function 1", f_shift="shift_data_23.txt", f_matrix="M_23_D", f_bias=2300):
        xichma = array([10, 20, 30, 40, 50])
        lamda = array([1, 1e-6, 1e-26, 1e-6, 1e-6])
        bias = array([0, 100, 200, 300, 400])
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_2__(f_shift, f_matrix)

        # 1. Rotated Rosenbrock’s Function F4’
        z = self.solution - shift[0]
        g1 = lamda[0] * self.rosenbrock__(dot(matrix[:self.problem_size, :], z)) + bias[0]
        w1 = (1.0 / sqrt(sum(z ** 2))) * exp(-sum(z ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # 2. High Conditioned Elliptic Function F1’
        t2 = self.solution - shift[1]
        g2 = lamda[1] * self.elliptic__(solution=self.solution) + bias[1]
        w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # 3. Rotated Bent Cigar Function F2’
        t3 = self.solution - shift[2]
        g3 = lamda[2] * self.bent_cigar__(dot(matrix[2 * self.problem_size: 3 * self.problem_size, :], t3)) + bias[2]
        w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        # 4. Rotated Discus Function F3’
        t4 = self.solution - shift[3]
        g4 = lamda[3] * self.discus__(dot(matrix[3 * self.problem_size: 4 * self.problem_size, :], t4)) + bias[3]
        w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * self.problem_size * xichma[3] ** 2))

        # 4. High Conditioned Elliptic Function F1’
        t5 = self.solution - shift[4]
        g5 = lamda[4] * self.elliptic__(self.solution) + bias[4]
        w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * self.problem_size * xichma[4] ** 2))

        sw = sum([w1, w2, w3, w4, w5])
        result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
        return result + f_bias

    def F24(self, name="Composition Function 2", f_shift="shift_data_24.txt", f_matrix="M_24_D", f_bias=2400):
        xichma = array([20, 20, 20])
        lamda = array([1, 1, 1])
        bias = array([0, 100, 200])
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_2__(f_shift, f_matrix)

        # 1. Schwefel's Function F10'
        t1 = self.solution - shift[0]
        g1 = lamda[0] * self.modified_schwefel__(self.solution) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # 2. Rotated Rastrigin’s Function F9’
        t2 = self.solution - shift[1]
        g2 = lamda[1] * self.rastrigin__(dot(matrix[self.problem_size: 2 * self.problem_size, :], t2)) + bias[1]
        w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # 3. Rotated HGBat Function F14’
        t3 = self.solution - shift[2]
        g3 = lamda[2] * self.hgbat__(dot(matrix[2 * self.problem_size: 3 * self.problem_size, :], t3)) + bias[2]
        w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        sw = sum([w1, w2, w3])
        result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
        return result + f_bias

    def F25(self, name="Composition Function 3", f_shift="shift_data_25.txt", f_matrix="M_25_D", f_bias=2500):
        xichma = array([10, 30, 50])
        lamda = array([0.25, 1, 1e-7])
        bias = array([0, 100, 200])
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_2__(f_shift, f_matrix)

        # 1. Rotated Schwefel's Function F11’
        t1 = self.solution - shift[0]
        g1 = lamda[0] * self.modified_schwefel__(dot(matrix[:self.problem_size, :], t1)) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # 2. Rotated Rastrigin’s Function F9’
        t2 = self.solution - shift[1]
        g2 = lamda[1] * self.rastrigin__(dot(matrix[self.problem_size: 2 * self.problem_size, :], t2)) + bias[1]
        w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # 3. Rotated High Conditioned Elliptic Function F1'
        t3 = self.solution - shift[2]
        g3 = lamda[2] * self.elliptic__(dot(matrix[2 * self.problem_size: 3 * self.problem_size, :], t3)) + bias[2]
        w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        sw = sum([w1, w2, w3])
        result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
        return result + f_bias

    def F26(self, name="Composition Function 4", f_shift="shift_data_26.txt", f_matrix="M_26_D", f_bias=2600):
        xichma = array([10, 10, 10, 10, 10])
        lamda = array([0.25, 1, 1e-7, 2.5, 10])
        bias = array([0, 100, 200, 300, 400])
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_2__(f_shift, f_matrix)

        # 1. Rotated Schwefel's Function F11’
        t1 = self.solution - shift[0]
        g1 = lamda[0] * self.modified_schwefel__(dot(matrix[:self.problem_size, :], t1)) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # 2. Rotated HappyCat Function F13’
        t2 = self.solution - shift[1]
        g2 = lamda[1] * self.happy_cat__(dot(matrix[self.problem_size:2 * self.problem_size, :], t2)) + bias[1]
        w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # 3. Rotated High Conditioned Elliptic Function F1’
        t3 = self.solution - shift[2]
        g3 = lamda[2] * self.elliptic__(dot(matrix[2 * self.problem_size: 3 * self.problem_size, :], t3)) + bias[2]
        w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        # 4. Rotated Weierstrass Function F6’
        t4 = self.solution - shift[3]
        g4 = lamda[3] * self.weierstrass__(dot(matrix[3 * self.problem_size: 4 * self.problem_size, :], t4)) + bias[3]
        w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * self.problem_size * xichma[3] ** 2))

        # 5. Rotated Griewank’s Function F7’
        t5 = self.solution - shift[4]
        g5 = lamda[4] * self.griewank__(dot(matrix[4 * self.problem_size:, :], t5)) + bias[4]
        w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * self.problem_size * xichma[4] ** 2))

        sw = sum([w1, w2, w3, w4, w5])
        result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
        return result + f_bias


    def F27(self, name="Composition Function 5", f_shift="shift_data_27.txt", f_matrix="M_27_D", f_bias=2700):
        xichma = array([10, 10, 10, 20, 20])
        lamda = array([10, 10, 2.5, 25, 1e-6])
        bias = array([0, 100, 200, 300, 400])
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_2__(f_shift, f_matrix)

        # 1. Rotated HGBat Function F14'
        t1 = self.solution - shift[0]
        g1 = lamda[0] * self.hgbat__(dot(matrix[:self.problem_size, :], t1)) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # 2. Rotated Rastrigin’s Function F9’
        t2 = self.solution - shift[1]
        g2 = lamda[1] * self.rastrigin__(dot(matrix[self.problem_size:2 * self.problem_size, :], t2)) + bias[1]
        w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # 3. Rotated Schwefel's Function F11’
        t3 = self.solution - shift[2]
        g3 = lamda[2] * self.modified_schwefel__(dot(matrix[2 * self.problem_size: 3 * self.problem_size, :], t3)) + bias[2]
        w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        # 4. Rotated Weierstrass Function F6’
        t4 = self.solution - shift[3]
        g4 = lamda[3] * self.weierstrass__(dot(matrix[3 * self.problem_size: 4 * self.problem_size, :], t4)) + bias[3]
        w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * self.problem_size * xichma[3] ** 2))

        # 5. Rotated High Conditioned Elliptic Function F1’
        t5 = self.solution - shift[4]
        g5 = lamda[4] * self.elliptic__(dot(matrix[4 * self.problem_size:, :], t5)) + bias[4]
        w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * self.problem_size * xichma[4] ** 2))

        sw = sum([w1, w2, w3, w4, w5])
        result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
        return result + f_bias

    def F28(self, name="Composition Function 6", f_shift="shift_data_28.txt", f_matrix="M_28_D", f_bias=2800):
        xichma = array([10, 20, 30, 40, 50])
        lamda = array([2.5, 10, 2.5, 5e-4, 1e-6])
        bias = array([0, 100, 200, 300, 400])
        f_matrix = f_matrix + str(self.problem_size) + ".txt"
        shift, matrix = self.__load_shift_matrix_2__(f_shift, f_matrix)

        # 1. Rotated Expanded Griewank’s plus Rosenbrock’s Function F15’
        t1 = self.solution - shift[0]
        g1 = lamda[0] * self.F15(f_bias=0) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # 2. Rotated HappyCat Function F13’
        t2 = self.solution - shift[1]
        g2 = lamda[1] * self.happy_cat__(dot(matrix[self.problem_size:2 * self.problem_size, :], t2)) + bias[1]
        w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # 3. Rotated Schwefel's Function F11’
        t3 = self.solution - shift[2]
        g3 = lamda[2] * self.modified_schwefel__(dot(matrix[2 * self.problem_size: 3 * self.problem_size, :], t3)) + bias[2]
        w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        # 4. Rotated Expanded Scaffer’s F6 Function F16’
        t4 = self.solution - shift[3]
        g4 = lamda[3] * self.expanded_scaffer__(dot(matrix[3 * self.problem_size: 4 * self.problem_size, :], t4)) + bias[3]
        w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * self.problem_size * xichma[3] ** 2))

        # 5. Rotated High Conditioned Elliptic Function F1’
        t5 = self.solution - shift[4]
        g5 = lamda[4] * self.elliptic__(dot(matrix[4 * self.problem_size:, :], t5)) + bias[4]
        w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * self.problem_size * xichma[4] ** 2))

        sw = sum([w1, w2, w3, w4, w5])
        result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
        return result + f_bias

    def F29(self, name="Composition Function 7", f_shift="shift_data_29.txt", f_matrix="M_29_D", f_bias=2900):
        num_funcs = 3
        func_names = ["F17", "F18", "F19"]
        xichma = array([10, 30, 50])
        lamda = array([1, 1, 1])
        bias = array([0, 100, 200])
        shift = self.load_matrix_data__(f_shift)[:, :self.problem_size]

        weights = ones(num_funcs)
        fits = ones(num_funcs)
        for i in range(0, num_funcs):
            func = self.__getattribute__(func_names[i])
            t1 = lamda[i] * func(f_shift=f_shift, f_matrix=f_matrix, shuffle=29, f_bias=0) + bias[i]
            fits[i] = t1
            temp = self.solution - shift[i]
            weights[i] = (1.0 / sqrt(sum(temp ** 2))) * exp(-sum(temp ** 2) / (2 * self.problem_size * xichma[i] ** 2))
        sw = sum(weights)
        result = 0.0
        for i in range(0, num_funcs):
            result += (weights[i] / sw) * fits[i]
        return result + f_bias

    def F30(self, name="Composition Function 8", f_shift="shift_data_30.txt", f_matrix="M_30_D", f_bias=3000):
        num_funcs = 3
        func_names = ["F20", "F21", "F22"]
        xichma = array([10, 30, 50])
        lamda = array([1, 1, 1])
        bias = array([0, 100, 200])
        shift = self.load_matrix_data__(f_shift)[:,:self.problem_size]

        weights = ones(num_funcs)
        fits = ones(num_funcs)
        for i in range(0, num_funcs):
            func = self.__getattribute__(func_names[i])
            t1 = lamda[i] * func(f_shift=f_shift, f_matrix=f_matrix, shuffle=29, f_bias=0) + bias[i]
            fits[i] = t1
            temp = self.solution - shift[i]
            weights[i] = (1.0 / sqrt(sum(temp ** 2))) * exp(-sum(temp ** 2) / (2 * self.problem_size * xichma[i] ** 2))
        sw = sum(weights)
        result = 0.0
        for i in range(0, num_funcs):
            result += (weights[i] / sw) * fits[i]
        return result + f_bias




#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 16:18, 20/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
# -------------------------------------------------------------------------------------------------------%

from opfunu.cec.cec2005.root import Root
from numpy import cos


class Model(Root):
    def __init__(self, f_name="Shifted Expanded Griewank's plus Rosenbrock's Function (F8F2)", f_shift_data_file="data_EF8F2",
                 f_ext='.txt', f_bias=-130):
        Root.__init__(self, f_name, f_shift_data_file, f_ext, f_bias)

    def __f8__(self, x=None):
        return x ** 2 / 4000 - cos(x) + 1

    def __f2__(self, x=None):
        return 100 * (x[0] ** 2 - x[1]) ** 2 + (x[0] - 1) ** 2

    def _main__(self, solution=None):
        problem_size = len(solution)
        if problem_size > 100:
            print("CEC 2005 not support for problem size > 100")
            return 1
        shift_data = self.load_shift_data()[:problem_size]
        z = solution - shift_data + 1

        result = 0
        for i in range(0, problem_size):
            if i == problem_size - 1:
                result += self.__f8__(self.__f2__([z[i], z[0]]))
            else:
                result += self.__f8__(self.__f2__(z[i:i + 2]))
        return result + self.f_bias



#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 16:19, 24/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from numpy.random import seed, permutation
from numpy import dot, ones, array, ceil
from opfunu.cec.cec2015.utils import *

SUPPORT_DIMENSION = [10, 30]


def F1(solution=None, name="Rotated Bent Cigar Function", f_shift_file="shift_data_1_D", f_matrix_file="M_1_D", bias=100):
    problem_size = len(solution)
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    z = dot(solution - shift_data, matrix)
    return f1_bent_cigar__(z) + bias


def F2(solution=None, name="Rotated Discus Function", f_shift_file="shift_data_2_D", f_matrix_file="M_2_D", bias=200):
    problem_size = len(solution)
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    z = dot(solution - shift_data, matrix)
    return f2_discus__(z) + bias


def F3(solution=None, name="Shifted and Rotated Weierstrass Function", f_shift_file="shift_data_3_D", f_matrix_file="M_3_D", bias=300):
    problem_size = len(solution)
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    z = dot(0.5*(solution - shift_data) / 100, matrix)
    return f3_weierstrass__(z) + bias


def F4(solution=None, name="Shifted and Rotated Schwefel’s Function", f_shift_file="shift_data_4_D", f_matrix_file="M_4_D", bias=400):
    problem_size = len(solution)
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    z = dot(1000 * (solution - shift_data) / 100, matrix)
    return f4_modified_schwefel__(z) + bias


def F5(solution=None, name="Shifted and Rotated Katsuura Function", f_shift_file="shift_data_5_D", f_matrix_file="M_5_D", bias=500):
    problem_size = len(solution)
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    z = dot(5 * (solution - shift_data) / 100, matrix)
    return f5_katsuura__(z) + bias


def F6(solution=None, name="Shifted and Rotated HappyCat Function", f_shift_file="shift_data_6_D", f_matrix_file="M_6_D", bias=600):
    problem_size = len(solution)
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    z = dot(5 * (solution - shift_data) / 100, matrix)
    return f6_happy_cat__(z) + bias


def F7(solution=None, name="Shifted and Rotated HGBat Function", f_shift_file="shift_data_7_D", f_matrix_file="M_7_D", bias=700):
    problem_size = len(solution)
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    z = dot(5 * (solution - shift_data) / 100, matrix)
    return f7_hgbat__(z) + bias


def F8(solution=None, name="Shifted and Rotated Expanded Griewank’s plus Rosenbrock’s Function",
       f_shift_file="shift_data_8_D", f_matrix_file="M_8_D", bias=800):
    problem_size = len(solution)
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    z = dot(5 * (solution - shift_data) / 100, matrix) + 1
    return f8_expanded_griewank__(z) + bias


def F9(solution=None, name="Shifted and Rotated Expanded Scaffer’s F6 Function",
       f_shift_file="shift_data_9_D", f_matrix_file="M_9_D", bias=900):
    problem_size = len(solution)
    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    z = dot(solution - shift_data, matrix) + 1
    return f9_expanded_scaffer__(z) + bias


def F10(solution=None, name="Hybrid Function 1 (N=3)", f_shift_file="shift_data_10_D", f_matrix_file="M_10_D",
        f_shuffle_file="shuffle_data_10_D", bias=1000):
    problem_size = len(solution)
    p = array([0.3, 0.3, 0.4])
    n1 = int(ceil(p[0] * problem_size))
    n2 = int(ceil(p[1] * problem_size))

    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
        f_shuffle = f_shuffle_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    shuffle = (load_shift_data__(f_shuffle) - ones(problem_size)).astype(int)
    idx1 = shuffle[:n1]
    idx2 = shuffle[n1:(n1 + n2)]
    idx3 = shuffle[(n1 + n2):]
    mz = dot(solution - shift_data, matrix)
    return f4_modified_schwefel__(mz[idx1]) + f12_rastrigin__(mz[idx2]) + f13_elliptic__(mz[idx3]) + bias


def F11(solution=None, name="Hybrid Function 2 (N=4)", f_shift_file="shift_data_11_D", f_matrix_file="M_11_D",
        f_shuffle_file="shuffle_data_11_D", bias=1100):
    problem_size = len(solution)
    p = array([0.2, 0.2, 0.3, 0.3])
    n1 = int(ceil(p[0] * problem_size))
    n2 = int(ceil(p[1] * problem_size))
    n3 = int(ceil(p[2] * problem_size))

    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
        f_shuffle = f_shuffle_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    shuffle = (load_shift_data__(f_shuffle) - ones(problem_size)).astype(int)
    idx1 = shuffle[:n1]
    idx2 = shuffle[n1:(n1 + n2)]
    idx3 = shuffle[(n1 + n2):(n1+n2+n3)]
    idx4 = shuffle[(n1 + n2 + n3):]
    mz = dot(solution - shift_data, matrix)
    return f11_griewank__(mz[idx1]) + f3_weierstrass__(mz[idx2]) + f10_rosenbrock__(mz[idx3]) + f9_expanded_scaffer__(mz[idx4]) + bias


def F12(solution=None, name="Hybrid Function 3 (N=5)", f_shift_file="shift_data_12_D", f_matrix_file="M_12_D",
        f_shuffle_file="shuffle_data_12_D", bias=1200):
    problem_size = len(solution)
    p = array([0.1, 0.2, 0.2, 0.2, 0.3])
    n1 = int(ceil(p[0] * problem_size))
    n2 = int(ceil(p[1] * problem_size))
    n3 = int(ceil(p[2] * problem_size))
    n4 = int(ceil(p[3] * problem_size))

    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
        f_shuffle = f_shuffle_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)
    shuffle = (load_shift_data__(f_shuffle) - ones(problem_size)).astype(int)
    idx1 = shuffle[:n1]
    idx2 = shuffle[n1:(n1 + n2)]
    idx3 = shuffle[(n1 + n2):(n1 + n2 + n3)]
    idx4 = shuffle[(n1 + n2 + n3):(n1+n2+n3+n4)]
    idx5 = shuffle[(n1+n2+n3+n4):]
    mz = dot(solution - shift_data, matrix)
    return f5_katsuura__(mz[idx1]) + f6_happy_cat__(mz[idx2]) + f8_expanded_griewank__(mz[idx3]) + f4_modified_schwefel__(mz[idx4]) + \
           f14_ackley__(mz[idx5]) + bias


def F13(solution=None, name="Composition Function 1 (N=5)", f_shift_file="shift_data_13_D", f_matrix_file="M_13_D", f_bias=1300):
    problem_size = len(solution)
    xichma = array([10, 20, 30, 40, 50])
    lamda = array([1, 1e-6, 1e-26, 1e-6, 1e-6])
    bias = array([0, 100, 200, 300, 400])

    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)

    # 1. Rotated Rosenbrock’s Function F10
    t1 = solution - shift_data[:problem_size]
    g1 = lamda[0] * f10_rosenbrock__(dot(t1, matrix[:problem_size, :])) + bias[0]
    w1 = (1.0 / sqrt(sum(t1**2))) * exp(-sum(t1**2) / (2*problem_size*xichma[0]**2))

    # 2. High Conditioned Elliptic Function
    t2 = solution - shift_data[problem_size:2*problem_size]
    g2 = lamda[1] * f13_elliptic__(solution) + bias[1]
    w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * problem_size * xichma[1] ** 2))

    # 3. Rotated Bent Cigar Function f1
    t3 = solution - shift_data[2*problem_size:3*problem_size]
    g3 = lamda[2] * f1_bent_cigar__(dot(t3, matrix[2*problem_size: 3*problem_size, :])) + bias[2]
    w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * problem_size * xichma[2] ** 2))

    # 4. Rotated Discus Function f2
    t4 = solution - shift_data[3 * problem_size: 4 * problem_size]
    g4 = lamda[3] * f2_discus__(dot(t4, matrix[problem_size*2:3*problem_size, :]) ) + bias[3]
    w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * problem_size * xichma[3] ** 2))

    # 4. High Conditioned Elliptic Function f13
    t5 = solution - shift_data[4 * problem_size:]
    g5 = lamda[4] * f13_elliptic__(solution) + bias[4]
    w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * problem_size * xichma[4] ** 2))

    sw = sum([w1, w2, w3, w4, w5])
    result = (w1*g1 + w2*g2 + w3*g3 + w4*g4 + w5*g5) / sw
    return result + f_bias


def F14(solution=None, name="Composition Function 2 (N=3)", f_shift_file="shift_data_14_D", f_matrix_file="M_14_D", f_bias=1400):
    problem_size = len(solution)
    xichma = array([10, 30, 50])
    lamda = array([0.25, 1, 1e-7])
    bias = array([0, 100, 200])

    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)

    # 1. Rotated Schwefel's Function f4
    t1 = solution - shift_data[:problem_size]
    g1 = lamda[0] * f4_modified_schwefel__(dot(t1, matrix[:problem_size, :])) + bias[0]
    w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * problem_size * xichma[0] ** 2))

    # 2. Rotated Rastrigin’s Function f12
    t2 = solution - shift_data[problem_size:2 * problem_size]
    g2 = lamda[1] * f12_rastrigin__(dot(t2, matrix[problem_size:2*problem_size, :])) + bias[1]
    w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * problem_size * xichma[1] ** 2))

    # 3. Rotated High Conditioned Elliptic Function f13
    t3 = solution - shift_data[2 * problem_size:3 * problem_size]
    g3 = lamda[2] * f13_elliptic__(dot(t3, matrix[2 * problem_size: 3 * problem_size, :])) + bias[2]
    w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * problem_size * xichma[2] ** 2))

    sw = sum([w1, w2, w3])
    result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
    return result + f_bias


def F15(solution=None, name="Composition Function 3 (N=5)", f_shift_file="shift_data_15_D", f_matrix_file="M_15_D", f_bias=1500):
    problem_size = len(solution)
    xichma = array([10, 10, 10, 20, 20])
    lamda = array([10, 10, 2.5, 25, 1e-6])
    bias = array([0, 100, 200, 300, 400])

    if problem_size in SUPPORT_DIMENSION:
        f_matrix = f_matrix_file + str(problem_size) + ".txt"
        f_shift = f_shift_file + str(problem_size) + ".txt"
    else:
        print("CEC 2015 function only support problem size 10, 30")
        return 1
    shift_data = load_shift_data__(f_shift)
    matrix = load_matrix_data__(f_matrix)

    # 1. Rotated HGBat Function f7
    t1 = solution - shift_data[:problem_size]
    g1 = lamda[0] * f7_hgbat__(dot(t1, matrix[:problem_size, :])) + bias[0]
    w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * problem_size * xichma[0] ** 2))

    # 2. Rotated Rastrigin’s Function f12
    t2 = solution - shift_data[problem_size:2 * problem_size]
    g2 = lamda[1] * f12_rastrigin__(dot(t2, matrix[problem_size:2*problem_size, :])) + bias[1]
    w2 = (1.0 / sqrt(sum(t2 ** 2))) * exp(-sum(t2 ** 2) / (2 * problem_size * xichma[1] ** 2))

    # 3. Rotated Schwefel's Function f4
    t3 = solution - shift_data[2 * problem_size:3 * problem_size]
    g3 = lamda[2] * f4_modified_schwefel__(dot(t3, matrix[2 * problem_size: 3 * problem_size, :])) + bias[2]
    w3 = (1.0 / sqrt(sum(t3 ** 2))) * exp(-sum(t3 ** 2) / (2 * problem_size * xichma[2] ** 2))

    # 4. Rotated Weierstrass Function f3
    t4 = solution - shift_data[3 * problem_size: 4 * problem_size]
    g4 = lamda[3] * f3_weierstrass__(dot(t4, matrix[3*problem_size: 4* problem_size, :])) + bias[3]
    w4 = (1.0 / sqrt(sum(t4 ** 2))) * exp(-sum(t4 ** 2) / (2 * problem_size * xichma[3] ** 2))

    # 4. Rotated High Conditioned Elliptic Function f13
    t5 = solution - shift_data[4 * problem_size:]
    g5 = lamda[4] * f13_elliptic__(dot(t5, matrix[4 * problem_size:, :])) + bias[4]
    w5 = (1.0 / sqrt(sum(t5 ** 2))) * exp(-sum(t5 ** 2) / (2 * problem_size * xichma[4] ** 2))

    sw = sum([w1, w2, w3, w4, w5])
    result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
    return result + f_bias




#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 02:18, 22/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

import numpy as np
from opfunu.cec.cec2014.function import *
from opfunu.cec.cec2014.unconstraint2 import Model as MD2
from opfunu.cec.cec2014.unconstraint import Model as MD

problem_size = 10
solution = np.random.uniform(0, 1, problem_size)


result1 = F1(solution)
print(result1)

result2 = MD2(solution)
print(result2.F1())

result3 = MD(problem_size)
print(result3.F1(solution))









#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 14:50, 20/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from opfunu.cec.cec2005.root import Root
from numpy import dot, max, abs, array
from pandas import read_csv


class Model(Root):
    def __init__(self, f_name="Schwefel's Problem 2.6 with Global Optimum on Bounds", f_shift_data_file="data_schwefel_206",
                 f_ext='.txt', f_bias=-310):
        Root.__init__(self, f_name, f_shift_data_file, f_ext, f_bias)

    def load_shift_data(self):
        data = read_csv(self.support_path_data + self.f_shift_data_file + self.f_ext, delimiter='\s+', index_col=False, header=None)
        data = data.values
        shift_data = data[:1, :]
        matrix_data = data[1:, :]
        return shift_data, matrix_data

    def _main__(self, solution=None):
        problem_size = len(solution)
        if problem_size > 100:
            print("CEC 2005 not support for problem size > 100")
            return 1
        shift_data, matrix_data = self.load_shift_data()
        shift_data = shift_data.reshape(-1)[:problem_size]
        t1 = int(0.25 * problem_size) + 1
        t2 = int(0.75 * problem_size)
        shift_data[:t1] = -100
        shift_data[t2:] = 100
        matrix_data = matrix_data[:problem_size, :problem_size]

        result = []
        for i in range(0, problem_size):
            temp = abs(dot(matrix_data[i], solution) - dot(matrix_data[i], shift_data))
            result.append(temp)
        return max(array(result)) + self.f_bias





#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 16:07, 20/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from opfunu.cec.cec2005.root import Root
from numpy import dot, sin, cos
from pandas import read_csv


class Model(Root):
    def __init__(self, f_name="Schwefel's Problem 2.13", f_shift_data_file="data_schwefel_213",
                 f_ext='.txt', f_bias=-460):
        Root.__init__(self, f_name, f_shift_data_file, f_ext, f_bias)

    def load_shift_data(self):
        data = read_csv(self.support_path_data + self.f_shift_data_file + self.f_ext, delimiter='\s+', index_col=False, header=None)
        data = data.values
        a_matrix = data[:100, :]
        b_matrix = data[100:200, :]
        shift_data = data[200:, :]
        return shift_data, a_matrix, b_matrix

    def _main__(self, solution=None):
        problem_size = len(solution)
        if problem_size > 100:
            print("CEC 2005 not support for problem size > 100")
            return 1
        shift_data, a_matrix, b_matrix = self.load_shift_data()
        shift_data = shift_data.reshape(-1)[:problem_size]
        a_matrix = a_matrix[:problem_size, :problem_size]
        b_matrix = b_matrix[:problem_size, :problem_size]

        result = 0.0
        for i in range(0, problem_size):
            t1 = dot(a_matrix[i], sin(shift_data)) + dot(b_matrix[i], cos(shift_data))
            t2 = dot(a_matrix[i], sin(solution)) + dot(b_matrix[i], cos(solution))
            result += (t1-t2)**2
        return result + self.f_bias




#!/usr/bin/env python                                                                                   #
# ------------------------------------------------------------------------------------------------------#
# Created by "Thieu Nguyen" at 02:52, 07/12/2019                                                        #
#                                                                                                       #
#       Email:      nguyenthieu2102@gmail.com                                                           #
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  #
#       Github:     https://github.com/thieunguyen5991                                                  #
#-------------------------------------------------------------------------------------------------------#


import numpy as np

class Functions:
    """
        This class of functions is belongs to 1-dimensional space
    """

    def _gramacy_lee__(self, solution=None):
        """
        Class: uni-modal, non-convex, continuous
        Global: 1 global minimum fx = −0.869011134989500, atx 0.548563444114526
        Link: http://benchmarkfcns.xyz/benchmarkfcns/gramacyleefcn.html

        @param solution: A numpy array include 1 items like: [0.5], limited range: [-0.5, 2.5]

        """
        n = len(solution)
        assert (n == 1, 'Gramacy and Lee function is only defined on a 1D space.')
        return np.sin(10*np.pi*solution) / (2*solution) + (solution-1)**4
